use crate::errors::BASE_BLACKJACK_ERROR_CODE;

/// Blackjack errors
#[repr(C)]
#[derive(
    Debug, ::failure::Fail, Copy, Clone, serde::Serialize, serde::Deserialize, Eq, PartialEq, Hash,
)]
pub enum BlackJackError {
    /// Player has lost the game, no other actions can be made. (2001)
    #[fail(display = "Player has lost the game, no other actions can be made.")]
    PlayerLostError,
    /// Player has won the game, no other actions can be made. (2002)
    #[fail(display = "Player has won the game, no other actions can be made.")]
    PlayerWonError,
    /// Game Ran out of cards (2003)
    #[fail(display = "The game has run out of cards in the deck.")]
    OutOfCardsError,
}

impl crate::errors::ErrorCode for BlackJackError {
    fn error_code(&self) -> i32 {
        BASE_BLACKJACK_ERROR_CODE
            + match *self {
                BlackJackError::PlayerLostError => 1,
                BlackJackError::PlayerWonError => 2,
                BlackJackError::OutOfCardsError => 3,
            }
    }
}
