/// Alias for solitaire
pub type SError<T> = Result<T, SolitaireError>;

use crate::errors::{ErrorCode, BASE_SOLITAIRE_ERROR_CODE};

/// Solitaire Errors
#[repr(C)]
#[derive(
    Copy, Clone, Debug, ::failure::Fail, serde::Serialize, serde::Deserialize, Hash, PartialEq, Eq,
)]
pub enum SolitaireError {
    /// Invalid movement (3001)
    #[fail(display = "Illegal Movement")]
    IllegalMovementError,
    /// Out of range movement (3002)
    #[fail(display = "Request movement extends beyond any position cards lay")]
    OutOfRangeError,
    /// Out of cards, should be impossible (3003)
    #[fail(display = "Out of cards, this shouldn't be possible")]
    OutOfCardsError,
    /// Out of moves, exceeded u32 limit (3004)
    #[fail(display = "Out of moves, exceeded u32 limit")]
    OutOfMoves,
}

impl ErrorCode for SolitaireError {
    fn error_code(&self) -> i32 {
        BASE_SOLITAIRE_ERROR_CODE
            + match *self {
                SolitaireError::IllegalMovementError => 1,
                SolitaireError::OutOfRangeError => 2,
                SolitaireError::OutOfCardsError => 3,
                SolitaireError::OutOfMoves => 4,
            }
    }
}
